---
title: llarp/layers/platform/addr_mapper.hpp

---

# llarp/layers/platform/addr_mapper.hpp



## Namespaces

| Name           |
| -------------- |
| **[llarp](Namespaces/namespacellarp.md)** <br>[crypto.hpp] |
| **[llarp::layers](Namespaces/namespacellarp_1_1layers.md)**  |
| **[llarp::layers::flow](Namespaces/namespacellarp_1_1layers_1_1flow.md)**  |
| **[llarp::layers::platform](Namespaces/namespacellarp_1_1layers_1_1platform.md)**  |

## Classes

|                | Name           |
| -------------- | -------------- |
| struct | **[llarp::layers::platform::AddressMapping](Classes/structllarp_1_1layers_1_1platform_1_1AddressMapping.md)** <br>holds all info about a platform layer to flow layer mapping.  |
| class | **[llarp::layers::platform::AddressMappingEntry](Classes/classllarp_1_1layers_1_1platform_1_1AddressMappingEntry.md)** <br>container that holds an addressmapping and extra metadata we need in the addrmapper.  |
| class | **[llarp::layers::platform::AddrMapper](Classes/classllarp_1_1layers_1_1platform_1_1AddrMapper.md)** <br>in charge of mapping flow addresses and platform addresses to each other  |




## Source code

```cpp
#pragma once

#include "platform_addr.hpp"
#include <llarp/layers/flow/flow_info.hpp>
#include <llarp/net/ip_range_map.hpp>

#include <chrono>
#include <string>
#include <optional>
#include <unordered_map>
#include <utility>
#include <vector>

namespace llarp::layers::flow
{
  class FlowLayer;
}

namespace llarp::layers::platform
{

  struct AddressMapping
  {
    flow::FlowInfo flow_info;

    PlatformAddr src, dst;

    std::vector<IPRange> owned_ranges;

    bool
    owns_range(const IPRange& range) const;

    bool
    allows_ingres(const PlatformAddr& src, const PlatformAddr& dst) const;

    bool
    allows_egres(const PlatformAddr& src, const PlatformAddr& dst) const;

    std::string
    ToString() const;

    bool
    is_exit() const;

    AddressMapping() = default;
    AddressMapping(const AddressMapping&) = default;
    AddressMapping(AddressMapping&&) = default;

    AddressMapping&
    operator=(AddressMapping&&) = default;
    AddressMapping&
    operator=(const AddressMapping&) = default;

    bool
    operator==(const AddressMapping& other) const;
  };

  class AddressMappingEntry
  {
    using clock_t = std::chrono::steady_clock;
    clock_t::time_point _last_used_at;
    AddressMapping _entry;

   public:
    bool
    has_addr(const PlatformAddr& src, const PlatformAddr& dst) const;

    bool
    routes_addr(const PlatformAddr& dst) const;

    bool
    has_flow_info(const flow::FlowInfo& flow_info) const;

    inline auto
    idle_for() const noexcept
    {
      return clock_t::now() - _last_used_at;
    }

    constexpr const auto&
    last_used_at() const
    {
      return _last_used_at;
    }

    AddressMapping&
    access();

    const AddressMapping&
    view() const noexcept;
  };

  class AddrMapper
  {
    std::vector<AddressMappingEntry> _addrs;
    void
    remove_lru();

    PlatformAddr
    next_addr();

    friend class ReservedAddressMapping;

   public:
    const IPRange range;

    explicit AddrMapper(const IPRange& range);

    AddressMapping&
    put(AddressMapping&& ent);

    bool
    unmap(AddressMapping ent);

    void
    prune(flow::FlowLayer&);

    std::optional<AddressMapping>
    get_addr_for_flow(const flow::FlowInfo& flow);

    bool
    has_addr_for_flow(const flow::FlowInfo& flow) const;

    std::optional<AddressMapping>
    mapping_for(const PlatformAddr& src, const PlatformAddr& dst);

    std::vector<AddressMapping>
    mappings_to(const flow::FlowAddr& remote) const;

    bool
    is_full() const noexcept;

    AddressMapping&
    allocate_mapping(std::optional<PlatformAddr> src = std::nullopt);

    template <typename Viewer_t>
    void
    view_all_mappings(Viewer_t&& visit) const
    {
      for (const auto& ent : _addrs)
      {
        if (not visit(ent.view()))
          return;
      }
    }

    template <typename Predicate_t>
    [[nodiscard]] auto
    find_if(Predicate_t&& pred) noexcept
    {
      return std::find_if(
          _addrs.begin(), _addrs.end(), [pred](const auto& ent) { return pred(ent.view()); });
    }

    template <typename Predicate_t>
    [[nodiscard]] auto
    const_find_if(Predicate_t&& pred) const noexcept
    {
      return std::find_if(
          _addrs.cbegin(), _addrs.cend(), [pred](const auto& ent) { return pred(ent.view()); });
    }

    template <typename Predicate_t>
    void
    remove_if_entry(Predicate_t&& pred) noexcept
    {
      auto itr = _addrs.begin();
      while (itr != _addrs.end())
      {
        if (pred(*itr))
          itr = _addrs.erase(itr);
        else
          ++itr;
      }
    }

    template <typename Predicate_t>
    void
    remove_if(Predicate_t&& pred) noexcept
    {
      _addrs.erase(const_find_if(pred));
    }

    std::vector<AddressMapping>
    all_exits() const;

    net::IPRangeMap<flow::FlowAddr>
    exit_map() const;
  };

}  // namespace llarp::layers::platform

namespace llarp
{
  nlohmann::json
  to_json(const llarp::layers::platform::AddressMapping&);

  template <>
  inline constexpr bool IsToStringFormattable<layers::platform::AddressMapping> = true;
}  // namespace llarp
```


-------------------------------

Updated on 2023-04-27 at 04:17:17 +0000
